@@ -16,6 +16,7 @@
 import collections
 import gc
 import io
+import ipaddress
 import json
 import platform
 import socket
@@ -38,6 +39,7 @@ except ImportError:
     greenlet = None
 
 from oslo_middleware import base
+from oslo_middleware.exceptions import ConfigInvalid
 from oslo_middleware.healthcheck import opts
 
 
@@ -391,6 +393,16 @@ Reason
                                      group='healthcheck')
         self._path = self._conf_get('path')
         self._show_details = self._conf_get('detailed')
+        self._source_ranges = [
+            ipaddress.ip_network(r)
+            for r in self._conf_get('allowed_source_ranges')]
+        self._ignore_proxied_requests = self._conf_get(
+            'ignore_proxied_requests')
+
+        # (abhishekk): Verify that if `enable_by_files` and
+        # `disable_by_file` backends are not enabled at same time.
+        self._verify_configured_plugins()
+
         self._backends = stevedore.NamedExtensionManager(
             self.NAMESPACE, self._conf_get('backends'),
             name_order=True, invoke_on_load=True,
@@ -408,6 +420,15 @@ Reason
         self._default_accept = 'text/plain'
         self._ignore_path = False
 
+    def _verify_configured_plugins(self):
+        backends = self._conf_get('backends')
+        exclusive_plugins = ['disable_by_file', 'enable_by_files']
+
+        if set(exclusive_plugins).issubset(set(backends)):
+            raise ConfigInvalid('`enable_by_files` plugin and '
+                                '`disable_by_file` plugin should not be '
+                                'enabled at the same time.')
+
     def _conf_get(self, key, group='healthcheck'):
         return super(Healthcheck, self)._conf_get(key, group=group)
 
@@ -550,6 +571,24 @@ Reason
     def process_request(self, req):
         if not self._ignore_path and req.path != self._path:
             return None
+
+        if self._source_ranges:
+            remote_addr = ipaddress.ip_address(req.remote_addr)
+            for r in self._source_ranges:
+                if r.version == remote_addr.version and remote_addr in r:
+                    break
+            else:
+                # Because source ip is not included in allowed ranges, ignore
+                # the request in this middleware.
+                return None
+
+        if self._ignore_proxied_requests:
+            for hdr in [
+                'FORWARDED', 'FORWARDED_PROTO', 'FORWARDED_HOST',
+                'FORWARDED_FOR', 'FORWARDED_PREFIX']:
+                if req.environ.get("HTTP_X_%s" % hdr):
+                    return None
+
         results = [ext.obj.healthcheck(req.server_port)
                    for ext in self._backends]
         healthy = self._are_results_healthy(results)
